/*
 * $QNXLicenseC:
 * Copyright 2021, QNX Software Systems.
 * Copyright 2021, Texas Instruments Incorporated.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */
// #############################################################################
//
//  File drivers_usbwrapper.c
//
//  The usb wrapper functions
//
// #############################################################################
#include "startup.h"
#include "ti_sci.h"
#include "drivers_usbwrapper.h"
#include "J7200_pinmux.h"

#define J7200_CTRLMMR_PADCONFIG         0x11C000

// *****************************************************************************
//  Function Drivers_SetUsbPhyConfig
//
//  Configure the PHY configuration in static config wrapper MMRs.
// *****************************************************************************
uint32_t Drivers_SetUsbPhyConfig(uint8_t *baseaddr, int freq)
{
    uint32_t drvRet = 0;
    uint32_t pllRefSel = 0;

	/*
	 * USB3P0SS_STATIC_CONFIG: VBUS_SEL
	 * Configure the vbus_sel bit to 1 to select 1/3 voltage divider between
	 * USB2 Connector VBUS (5.25V) and the PHY VBUS pin.
	 */
	HW_Write32_BF(baseaddr, USB3P0SS_CMN_STATIC_CONFIG, 2, 1, 0x1);

	/*
	 * USB3P0SS_STATIC_CONFIG: LANE_REVERSE
	 * Configure the lane_reverse bit for D+/D- swapping
	 */
	HW_Write32And(baseaddr, USB3P0SS_CMN_STATIC_CONFIG, ~BIT(0));

    // Set the pll_ref_sel bits based on the HFOSC0 frequency
    //
    switch ( freq ) {
        case 19200: pllRefSel = 0x3; break;
        case 20000: pllRefSel = 0x4; break;
        case 24000: pllRefSel = 0x5; break;
        case 25000: pllRefSel = 0x6; break;
        case 26000: pllRefSel = 0x7; break;
        case 27000:
        default:
            // Error: Unknown frequency
            drvRet = 1;
            return drvRet;
    }

	/*
	 * USB3P0SS_STATIC_CONFIG: PLL_REF_SEL
	 * Indicates the frequency of the REF_CLOCK input used by the USB PLL
	 */
	HW_Write32_BF(baseaddr, USB3P0SS_CMN_STATIC_CONFIG, 8, 5, pllRefSel);

    return drvRet;
}

static int set_usb_clock()
{
    /* Set Clock */
#if 1
    // TI Update
    kprintf("%s:%d Not configuring SERDES clocks to avoid conflict with EthFW\n", __func__, __LINE__);
#else
    if (ti_sci_set_device_state(TISCI_DEV_SERDES_10G1, 0, MSG_DEVICE_SW_STATE_ON) == -1) {
        kprintf("%s:%d TISCI_DEV_SERDES_10G1 failed!\n", __func__, __LINE__);
        return (-1);
    }

    if (ti_sci_cmd_set_clk_parent(TISCI_DEV_SERDES_10G1, TISCI_DEV_SERDES_10G1_CORE_REF_CLK,
            TISCI_DEV_SERDES_10G1_CORE_REF_CLK_PARENT_HSDIV4_16FFT_MAIN_2_HSDIVOUT4_CLK) == -1) {
        kprintf("%s:%d TISCI_DEV_SERDES_10G1 failed!\n", __func__, __LINE__);
        return (-1);
    }

    if (ti_sci_set_clock_state(TISCI_DEV_SERDES_10G1, TISCI_DEV_SERDES_10G1_CORE_REF_CLK,
            0, MSG_CLOCK_SW_STATE_AUTO) == -1) {
        kprintf("%s:%d TISCI_DEV_SERDES_10G1 failed!\n", __func__, __LINE__);
        return (-1);
    }
#endif

    // USB sub-system clock, parent clock
    if (ti_sci_cmd_set_clk_parent(TISCI_DEV_USB0, TISCI_DEV_USB0_USB2_REFCLOCK_CLK,
            TISCI_DEV_USB0_USB2_REFCLOCK_CLK_PARENT_GLUELOGIC_HFOSC0_CLKOUT) == -1) {
        kprintf("%s:%d TISCI_DEV_USB0 failed!\n", __func__, __LINE__);
        return (-1);
    }

    if (ti_sci_set_clock_state(TISCI_DEV_USB0, TISCI_DEV_USB0_USB2_REFCLOCK_CLK,
            0, MSG_CLOCK_SW_STATE_AUTO) == -1) {
        kprintf("%s:%d TISCI_DEV_USB0 failed!\n", __func__, __LINE__);
        return (-1);
    }

	// verified
    if (ti_sci_set_clock_state(TISCI_DEV_USB0, TISCI_DEV_USB0_CLK_LPM_CLK,
            0, MSG_CLOCK_SW_STATE_AUTO) == -1) {
        kprintf("%s:%d TISCI_DEV_USB0 failed!\n", __func__, __LINE__);
        return (-1);
    }

	// verified
    if (ti_sci_set_device_state(TISCI_DEV_USB0, 0, MSG_DEVICE_SW_STATE_ON) == -1) {
        kprintf("%s:%d TISCI_DEV_USB0 failed!\n", __func__, __LINE__);
        return (-1);
    }

	return 0;
}

int init_usb_host(void)
{
#if 1
    // TI Update
#else
	uint8_t *baseaddr = (uint8_t *)CTRL_MMR0_CFG0_BASE;
#endif

    /* J7200 EVM only supports single USB interface, so the USB3.1 and USB2.0 cannot be supported simultaneously */
    /* Signal Mux Control USB2.0_MUX_SEL I2C0 ADDR: 0x21 */

	set_usb_clock();

    /* Set pinmux for USB0 */
    out32(J7200_CTRLMMR_PADCONFIG + PIN_USB0_DRVVBUS, PIN_PULL_DISABLE | PIN_MODE(0));  /* (T4) USB0_DRVVBUS */

#if 1
    // TI Update
    kprintf("%s:%d Not configuring SERDES to avoid conflict with EthFW\n", __func__, __LINE__);
#else
	/* TRM 916p */
	//HW_Write32Or(baseaddr, CTRLMMR_USB0_CTRL, SERDES_SEL_LN1);
	HW_Write32Or(baseaddr, CTRLMMR_USB0_CTRL, SERDES_SEL_LN3);
    /* 10G SERDES0 Config - spruiu1a pg 7844 */
	HW_Write32_BF(baseaddr, CTRLMMR_SERDES0_CLKSEL, 1, 0, MAIN_PLL2_HSDIV4_CLKOUT);

	HW_Write32_BF(baseaddr, CTRLMMR_SERDES0_LN1_CTRL, 1, 0, 0x2);
	HW_Write32_BF(baseaddr, CTRLMMR_SERDES0_LN3_CTRL, 1, 0, 0x2);
	/* Lane 0/2 is not defined for USB3 in J7200 TRM */
	HW_Write32_BF(baseaddr, CTRLMMR_SERDES0_LN0_CTRL, 1, 0, 0x2);
	HW_Write32_BF(baseaddr, CTRLMMR_SERDES0_LN2_CTRL, 1, 0, 0x2);
#endif

    /* PhyConfig */
    Drivers_SetUsbPhyConfig((uint8_t *)USB0_SUBSYS_BASE, sysFreqKhz);

    return (0);
}

#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn.ott.qnx.com/product/branches/7.1.0/trunk/hardware/startup/boards/j7200/evm/drivers_usbwrapper.c $ $Rev: 934596 $")
#endif
